package edu.uoc.tfc.j2ee.icampanario.components.authenticator;

import java.io.Serializable;

import javax.security.cert.X509Certificate;
import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.security.utils.Base64;

import edu.uoc.tfc.j2ee.icampanario.components.authenticator.exception.AuthenticatorException;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignTransaction;
import edu.uoc.tfc.j2ee.icampanario.components.signer.Signer;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignerFactory;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;

/**
 * 
 * Implementación del componente Authenticator que usa
 * el cliente de firma de la plataforma de firma electrónica
 * @firma para realizar la autenticación.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class AuthenticatorClienteAfirma implements Authenticator, Serializable {

	/**
	 * serial version uid
	 */
	private static final long serialVersionUID = 6663250556931071082L;
	
	/**
	 * Log
	 */
	private final static Log log = LogFactory.getLog(AuthenticatorClienteAfirma.class);
	
	/**
	 * Configuración
	 */
	private Configuration config;
	
	/**
	 * Constructor
	 * @param config Configuración
	 */
	public AuthenticatorClienteAfirma(Configuration config) {
		super();
		this.config = config;
	}


	/**
	 * Obtiene la url de autenticación
	 * @param request Petición que contiene los parámetros para realizar la autenticación
	 * @return URL de autenticación
	 * @throws AuthenticatorException
	 */
	@Override
	public String getAuthenticationURL(HttpServletRequest request) throws AuthenticatorException {
		String retorno = "";

		try {

			String idSesion = request.getSession().getId();

			if (idSesion == null || idSesion.equals("")) {
				log.error("session id is null or empty");
				throw new AuthenticatorException("session id is null or empty");
			}

			// realizamos una firma para obtener los datos del certificado digital del usuario
			Signer signer = SignerFactory.createInstance();
			SignTransaction signTransaction = new SignTransaction();

			SignDocument document = new SignDocument();
			document.setContent(idSesion.getBytes());

			signTransaction.addDocument(document);
			signer.initSign(signTransaction, request);

			String clientSignerConfig = (String) request.getAttribute("clientSignerConfig");

			if (clientSignerConfig == null || clientSignerConfig.equals("")) {
				log.error("clientSignerConfig is null or empty");
				throw new AuthenticatorException("clientSignerConfig is null or empty");
			}

			request.getSession().setAttribute("clientSignerConfig", clientSignerConfig);
			request.getSession().setAttribute("signTransAutenticacion", signTransaction);

			retorno = config.getString("url.return");

			if (retorno == null || retorno.equals("")) {
				log.warn("url.return of components.properties is empty");
			}
		}
		catch (Exception e) {
			log.error("Error in method getAuthenticationURL.", e);
			throw new AuthenticatorException(e);
		}

		return retorno;
	}

	/**
	 * Realiza la autenticación del usuario
	 * @param request Petición que contiene los parámetros para realizar la autenticación
	 * @return Usuario autenticado
	 * @throws AuthenticatorException
	 */
	@Override
	public User authenticate(HttpServletRequest request) throws AuthenticatorException {

		request.getSession().removeAttribute("clientSignerConfig");

		SignTransaction signTransaction = (SignTransaction)request.getSession().getAttribute("signTransAutenticacion");

		if (signTransaction == null) {
	        log.error("signTransAutenticacion is null or empty");
	        throw new AuthenticatorException("signTransAutenticacion is null or empty");
	    }

		request.getSession().removeAttribute("signTransAutenticacion");

		String csResult = (String)request.getParameter("clientSignerResult");

		if (csResult ==  null || csResult.equals("")) {
		    log.error("clientSignerResult is null or empty");
            throw new AuthenticatorException("clientSignerResult is null or empty");
		}

		JSONArray jsonArray = JSONArray.fromObject(csResult);

		if (jsonArray == null || jsonArray.isEmpty()) {
		    log.error("sign data (JSONArray) is null or empty");
            throw new AuthenticatorException("sign data (JSONArray) is null or empty");
		}

		String certificate = (String) ((JSONObject)jsonArray.get(0)).get("certificate");

		if (certificate == null || certificate.equals("")) {
			log.error("certificate is null o empty");
            throw new AuthenticatorException("certificate is null o empty");
		}

		//Creamos un usuario con los datos obtenidos del certificado digital (FNMT)
		User user = new User();
		user.setCertificate(certificate);
		
		try {
			
			// Tipo del certificado digital de usuario de la FNMT
			X509Certificate cert = X509Certificate.getInstance(Base64.decode(certificate));
			
			String sCert = cert.toString();
			String nombre = sCert.substring(sCert.indexOf("OID.1.3.6.1.4.1.5734.1.1=") + 25, sCert.indexOf(",", sCert.indexOf("OID.1.3.6.1.4.1.5734.1.1=") + 25));
			String apellido1 = sCert.substring(sCert.indexOf("OID.1.3.6.1.4.1.5734.1.2=") + 25, sCert.indexOf(",", sCert.indexOf("OID.1.3.6.1.4.1.5734.1.2=") + 25));
			String apellido2 = sCert.substring(sCert.indexOf("OID.1.3.6.1.4.1.5734.1.3=") + 25, sCert.indexOf(",", sCert.indexOf("OID.1.3.6.1.4.1.5734.1.3=") + 25));
			String nif = sCert.substring(sCert.indexOf("OID.1.3.6.1.4.1.5734.1.4=") + 25, sCert.indexOf("OID.1.3.6.1.4.1.5734.1.4=") + 34);
			
			user.setName(nombre);
			user.setSurname1(apellido1);
			user.setSurname2(apellido2);
			user.setIdNumber(nif);
			
		} catch (Exception e) {
			log.error("Error in method authenticate", e);
			throw new AuthenticatorException(e);
		}
		return user;
	}

}
