package edu.uoc.tfc.j2ee.icampanario.components.clientsigner.filter;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.tfc.j2ee.icampanario.components.clientsigner.exception.ClientSignerException;
import edu.uoc.tfc.j2ee.icampanario.components.clientsigner.resources.Resource;
import edu.uoc.tfc.j2ee.icampanario.components.clientsigner.resources.ResourceFactory;
import edu.uoc.tfc.j2ee.icampanario.components.clientsigner.util.RequestUtils;

/**
 * 
 * Filtro para poner disponible los recursos del cliente de @firma,
 * para ello obtiene los recursos (ficheros) que se encuentren en el 
 * ClassLoader en el paquete edu.uoc.tfc.j2ee.icampanario.components.clientsigner.static
 * En la aplicación se han empaquetado en la librería clientsigner.jar
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class FilterDispatcherClientSigner implements Filter {

		private final static Log log = LogFactory.getLog(FilterDispatcherClientSigner.class);

	    private String[] pathPrefixes;

	    private final Calendar lastModifiedCal = Calendar.getInstance();

	    public static final int TAM_BUFFER = 4096;

	    private FilterConfig filterConfig;

	    /**
	     * Inicializa el filtro indicando el directorio dónde están los
	     * recursos del cliente de @firma
	     *
	     * @param filterConfig Configuración del filtro
	     * @throws ServletException Si ocurre algún error.
	     */
	    public void init(FilterConfig filterConfig) throws ServletException {
	        this.filterConfig = filterConfig;

	        String param = filterConfig.getInitParameter("packages");
	        String packages = "edu.uoc.tfc.j2ee.icampanario.components.clientsigner.static";

	        if (param != null) {
	            packages = param;
	        }
	        this.pathPrefixes = parse(packages);
	    }

	    public void destroy() {
	    }

	    protected ServletContext getServletContext() {
	        return filterConfig.getServletContext();
	    }

	    protected FilterConfig getFilterConfig() {
	        return filterConfig;
	    }

	    /**
	     * Parsea los recursos mediante un carácter delimitador
	     *
	     * @param packages Paquete de recursos
	     * @return Array de recursos
	     */
	    protected String[] parse(String packages) {
	        if (packages == null) {
	            return null;
	        }
	        List<String> pathPrefixes = new ArrayList<String>();

	        StringTokenizer st = new StringTokenizer(packages, ", \n\t");
	        while (st.hasMoreTokens()) {
	            String pathPrefix = st.nextToken().replace('.', '/');
	            if (!pathPrefix.endsWith("/")) {
	                pathPrefix += "/";
	            }
	            pathPrefixes.add(pathPrefix);
	        }

	        return pathPrefixes.toArray(new String[pathPrefixes.size()]);
	    }

	    /**
	     * Realiza el filtro de todas las peticiones definidas en el mapeo en el web.xml
	     * para poner disponibles los recursos del cliente de @firma para que puedan ser
	     * usados en la vista.
	     */
	    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {

	        HttpServletRequest request = (HttpServletRequest) req;
	        HttpServletResponse response = (HttpServletResponse) res;
	        
	        String resourcePath = ""; 

	        try {

	            resourcePath = RequestUtils.getServletPath(request);

	            if ("".equals(resourcePath) && null != request.getPathInfo()) {
	                resourcePath = request.getPathInfo();
	            }

	            if (resourcePath.indexOf("/") != -1) {
	                if (resourcePath.indexOf("/common-js") != -1) {
	                    resourcePath = resourcePath.substring(resourcePath.lastIndexOf("/common-js"), resourcePath.length());
	                } else {
	                    resourcePath = resourcePath.substring(resourcePath.lastIndexOf("/"), resourcePath.length());
	                }
	            }

	            if (resourcePath.indexOf("/") != -1) {
	                String resourceAux = resourcePath.substring(resourcePath.lastIndexOf("/"), resourcePath.length());
	                try {
	                    if (!resourceAux.equals("/clientsigner-config.js")) {
	                        findStaticResource(resourcePath, request, response, chain);
	                    } else {
	                        findDinamicResource (resourcePath, request, response, chain);
	                    }
	                } catch (Exception e) {
                        chain.doFilter (request, response);
                    }
	            }

	            return;
	        } 
	        catch (Exception e) {
	        	if (e instanceof SocketException) {
	        		log.error("Se ha cancelado la carga del contenido estático: " + resourcePath);
	        	}
	        	if (e instanceof IllegalStateException) {
	        		log.error("Se ha cancelado la carga del contenido estático: " + resourcePath);
	        	}
	        	else {
	        		log.error("No se pudo servir contenido estático", e);
	        		throw new IOException(e);
	        	}
	        }
	    }

	    /**
	     * Carga un recurso dinámico en la respuesta de la petición
	     */
	    protected void findDinamicResource (String name, HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws IOException, ServletException, ClientSignerException {
	        if (!name.endsWith(".class")) {
	            for (String pathPrefix : pathPrefixes) {
	                Resource resource = ResourceFactory.createInstance();
	                InputStream is = resource.getResources(name, pathPrefix, request);
	                if (is != null) {

                        String contentType = getContentType(name);
                        if (contentType != null) {
                            response.setContentType(contentType);
                        }

                        response.setHeader("Cache-Control", "no-cache");
                        response.setHeader("Pragma", "no-cache");
                        response.setHeader("Expires", "-1");

                        copy(is, response.getOutputStream());

                        return;
	                }
	            }
	        }
            chain.doFilter(request, response);
	    }

	    /**
	     * Carga un recurso estático en la respuesta de la petición
	     */
	    protected void findStaticResource(String name, HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws IOException, ServletException {
	        if (!name.endsWith(".class")) {
	            for (String pathPrefix : pathPrefixes) {

	                InputStream is = findInputStream(name, pathPrefix);
	                if (is != null) {
	                    Calendar cal = Calendar.getInstance();

	                    long ifModifiedSince = 0;
	                    try {
	                        ifModifiedSince = request.getDateHeader("If-Modified-Since");
	                    } catch (Exception e) {
	                        log.warn("Invalid If-Modified-Since header value: '" + request.getHeader("If-Modified-Since") + "', ignoring");
	                    }
	                    long lastModifiedMillis = lastModifiedCal.getTimeInMillis();
	                    cal.add(Calendar.DAY_OF_MONTH, 1);
	                    long expires = cal.getTimeInMillis();

	                    if (ifModifiedSince > 0 && ifModifiedSince <= lastModifiedMillis) {
	                        response.setDateHeader("Expires", expires);
	                        response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
	                        is.close();
	                        return;
	                    }

	                    String contentType = getContentType(name);
	                    if (contentType != null) {
	                        response.setContentType(contentType);
	                    }

	                    response.setHeader("Cache-Control", "no-cache");
	                    response.setHeader("Pragma", "no-cache");
	                    response.setHeader("Expires", "-1");

	                    try {
	                        copy(is, response.getOutputStream());
	                    } 
	                    finally {
	                        is.close();
	                    }
	                    return;
	                }
	            }
	        }

	        chain.doFilter(request, response);
	    }

	    /**
	     * Devuelve el tipo mime en función de la extensión del recurso (fichero)
	     * @param name Nombre del recurso
	     * @return Tipo mime
	     */
	    protected String getContentType(String name) {
	        if (name.endsWith(".js")) {
	            return "text/javascript";
	        } else if (name.endsWith(".css")) {
	            return "text/css";
	        } else if (name.endsWith(".html")) {
	            return "text/html";
	        } else if (name.endsWith(".txt")) {
	            return "text/plain";
	        } else if (name.endsWith(".gif")) {
	            return "image/gif";
	        } else if (name.endsWith(".jpg") || name.endsWith(".jpeg")) {
	            return "image/jpeg";
	        } else if (name.endsWith(".png")) {
	            return "image/png";
	        } else if (name.endsWith(".cab")) {
	            return "application/cab";
	        } else if (name.endsWith(".jar")) {
	            return "application/java-archive";
	        } else if (name.endsWith(".zip")) {
	            return "application/zip";
	        } else if (name.endsWith(".gz")) {
	        	return "application/x-gzip";
	        } else if (name.endsWith(".jnlp")) {
	        	return "application/x-java-jnlp-file";
	        } else {
	            return null;
	        }
	    }

	    /**
	     * Realiza una copia del contenido del stream
	     * @param input Stream de entrada
	     * @param output Stream de salida
	     * @throws IOException
	     */
	    protected void copy(InputStream input, OutputStream output) throws IOException {
	        final byte[] buffer = new byte[TAM_BUFFER];
	        int n;
	        while (-1 != (n = input.read(buffer))) {
	            output.write(buffer, 0, n);
	        }
	        output.flush();
	    }

	    /**
	     * Busca un recurso en el classpath y lo devuelve
	     * @param name Nombre del recurso
	     * @param packagePrefix Prefijo del paquete
	     * @return Devuelve el recurso cargado en un InputStream 
	     * @throws IOException
	     */
	    protected InputStream findInputStream(String name, String packagePrefix) throws IOException {
	        String resourcePath;
	        if (packagePrefix.endsWith("/") && name.startsWith("/")) {
	            resourcePath = packagePrefix + name.substring(1);
	        } else {
	            resourcePath = packagePrefix + name;
	        }

	        return this.getClass().getClassLoader().getResourceAsStream(resourcePath);
	    }
	}

