package edu.uoc.tfc.j2ee.icampanario.components.signer;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.io.IOUtils;

/**
 * Documento para realizar la firma digital
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class SignDocument implements Serializable {

	/**
	 * serial version uid
	 */
	private static final long serialVersionUID = 4174088567926352260L;
	
    /**
     * Identificador del documento
     */
    private String id;

    /**
     * Contenido del documento
     */
    private byte[] content;

    /**
     * Contenido del documento como InputStream
     */
    private InputStream inputStream;

    /**
     * Nombre del documento
     */
    private String name;

    /**
     * Tipo mime o formato del documento
     */
    private String mimeType;

    /**
     * Hash del documento
     */
    private String hash;

    /**
     * Firma del documento
     */
    private String sign;

    /**
     * Otras propiedades del documento
     */
    private Map<String, Object> properties;

    /**
     * Constructor
     */
    public SignDocument() {
        properties = new HashMap<String, Object>();
    }

    /**
     * Devuelve el contenido del documento
     * @return Contenido del documento
     */
    public byte[] getContent() {
    	try {
	    	if (this.content == null && this.inputStream != null) {
	    		return IOUtils.toByteArray(this.inputStream);
	    	}
    	} catch (IOException e) {
			return content;
		}

    	return content;
    }

    /**
     * Establece el contenido del documento
     * @param content Contenido del documento
     */
    public void setContent(byte[] content) {
        this.content = content;
    }

    /**
     * Devuelve el nombre del documento
     * @return Nombre del documento
     */
    public String getName() {
        return name;
    }

    /**
     * Establece el nombre del documento
     * @param name Nombre del documento
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Devuelve el formato del documento
     * @return Formato del documento
     */
    public String getMimeType() {
        return mimeType;
    }

    /**
     * Establece el formato del documento
     * @param mimeType Formato del documento
     */
    public void setMimeType(String mimeType) {
        this.mimeType = mimeType;
    }

    /**
     * Devuelve el identificador del documento
     * @return Identificador del documento
     */
    public String getId() {
        return id;
    }

    /**
     * Establece el identificador del documento
     * @param id Identificador del documento
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * Devuelve la firma del documento
     * @return Firma del documento
     */
    public String getSign() {
        return sign;
    }

    /**
     * Establece la firma del documento
     * @param sign Firma del documento
     */
    public void setSign(String sign) {
        this.sign = sign;
    }

    /**
     * Devuelve otras propiedades del documento
     * @return Otras propiedades del documento
     */
    public Map<String, Object> getProperties() {
        return properties;
    }

    /**
     * Establece otras propiedades del documento
     * @param properties Otras propiedades del documento
     */
    public void setProperties(Map<String, Object> properties) {
        this.properties = properties;
    }

    /**
     * Devuelve una propiedad del documento
     * @param property Key de la propiedad del documento
     * @return Valor de la propiedad del documento
     */
    public Object getProperty(String property) {
        return properties.get(property);
    }

    /**
     * Establece una propiedad del documento
     * @param property Key de la propiedad del documento
     * @param value Valor de la propiedad del documento
     */
    public void setProperty(String property, Object value) {
        this.properties.put(property, value);
    }

    /**
     * Devuelve el hash del documento
     * @return Hash del documento
     */
    public String getHash() {
        return hash;
    }
    
    /**
     * Establece el hash del documento
     * @param hash Hash del documento
     */
    public void setHash(String hash) {
        this.hash = hash;
    }

    /**
     * Devuelve el contenido del documento como InputStream
     * @return Contenido del documento
     */
	public InputStream getInputStream() {
		if (this.inputStream != null) {
			return inputStream;
		} else if (this.content != null) {
			return new ByteArrayInputStream(content);
		}
		return this.inputStream;
	}

	/**
	 * Establece el contenido del documento como InputStream
	 * @param inputStream Contenido del documento como InputStream
	 */
	public void setInputStream(InputStream inputStream) {
		this.inputStream = inputStream;
	}


}
