package edu.uoc.tfc.j2ee.icampanario.components.signer;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.tfc.j2ee.icampanario.components.signer.exception.SignerException;

/**
 * 
 * Implementación del componente Signer que realiza la firma
 * digital de un documento usando el cliente de firma de @firma.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class SignerClienteAfirma implements Signer, Serializable {

	/**
	 * serial version uid
	 */
	private static final long serialVersionUID = -4564313984680003632L;
	
	/**
	 * log
	 */
	private final static Log log = LogFactory.getLog(SignerClienteAfirma.class);
	
	// Valores por defecto
	public static final String DEFAULT_SIGNATURE_FORMAT = "CMS";
	public static final String DEFAULT_HASH_ALGORITHM = "SHA1";
	public static final String DEFAULT_SIGN_DATA = "signature";
	public static final String DEFAULT_CERTIFICATE_DATA = "certificate";
	public static final String DEFAULT_TRANSACTION_DATA = "transactionId";

	/**
	 * Algoritmo usaro en la firma digital
	 */
	private String hashAlgoritm = DEFAULT_HASH_ALGORITHM;

	/**
	 * Formato de la firma digital
	 */
	private String signatureFormat = DEFAULT_SIGNATURE_FORMAT;

	/**
	 * Configuración
	 */
	Configuration config = null;
	
	/**
	 * Constructor
	 * @param config Configuración
	 */
    public SignerClienteAfirma(Configuration config) {
		super();
		this.config = config;
	}

    /**
	 * Inicializa la firma digital de documento
	 * @param signTransaction Transacción de firma
	 * @param request Petición de la que se recogen los parámetros para la firma digital
	 * @throws SignerException
	 */
	@Override
	public void initSign(SignTransaction signTransaction, HttpServletRequest request) throws SignerException {
		if (log.isDebugEnabled()) {
			log.debug("Init sign...");
		}

		try {

			for (Iterator<SignDocument> iterator = signTransaction.getDocuments().iterator(); iterator.hasNext();) {
				SignDocument document = iterator.next();
				if (this.hashAlgoritm.toUpperCase().startsWith("MD5")) {
					document.setHash(DigestUtils.md5Hex(document.getContent()));
				}
				else {
					document.setHash(DigestUtils.shaHex(document.getContent()));
				}
			}
		}
		catch (Exception e) {
			throw new SignerException(e);
		}

		//inicializamos los datos para el cliente de firma
        initSignerClientConfig(signTransaction, request);

	}

	/**
	 * Finaliza la firma digital del documento
	 * @param signTransaction Transacción de firma
	 * @param request Petición de la que se recogen los parámetros para la firma digital
	 * @throws SignerException
	 */
	@Override
	public void sign(SignTransaction signTransaction, HttpServletRequest request) throws SignerException {

		try {

			if (log.isDebugEnabled()) {
				log.debug("Finalize sign...");
			}

			String csResult = (String) request.getParameter("clientSignerResult");
	        if (csResult == null || csResult.equals("")) {
	            log.error ("clientSignerResult is null or empty");
	            throw new SignerException ("clientSignerResult is null or empty");
	        }

	        // usamos las librerías de JSON para tratar el resultado devuelto por el cliente de firma
	        JSONArray jsonArray = JSONArray.fromObject(csResult);
	        if (jsonArray == null) {
	            log.error("sign data (JSONArray) is null or empty");
	            throw new SignerException ("sign data (JSONArray) is null or empty");
	        }

	        for (Iterator<JSONObject> iterator = jsonArray.iterator(); iterator.hasNext();) {
				JSONObject jsonObject = iterator.next();

				String signature = (String) jsonObject.get(DEFAULT_SIGN_DATA);
		        if (signature == null || signature.equals("")) {
		            log.error("signature is null or empty");
		            throw new SignerException("signature is null or empty");
		        }

		        String certificate = (String) jsonObject.get(DEFAULT_CERTIFICATE_DATA);
		        if (certificate == null || certificate.equals("")) {
		        	log.error("certificate is null or empty");
		        	throw new SignerException("certificate is null or empty");
		        }

		        String userCertificate = signTransaction.getUser();
		        if (userCertificate != null && userCertificate.length() > 0 && !userCertificate.equals(certificate)) {
		        	log.error("the authenticated user is not the same as the signature ");
		        	throw new SignerException("the authenticated user is not the same as the signature");
		        }


		        String transaction = "";
		        if (jsonObject.get(DEFAULT_TRANSACTION_DATA) instanceof String) {
		        	transaction = (String)jsonObject.get(DEFAULT_TRANSACTION_DATA);
		        }
		        
		        if (transaction == null || transaction.length() == 0) {
		        	transaction = System.currentTimeMillis() + "";
		        }

		        if (log.isDebugEnabled()) {
		        	log.debug("transaction: " + transaction);
		        	log.debug("signature: " + signature);
		        	log.debug("certificate: " + certificate);
		        }
		        
		        //devolvemos en la propia transacción su id y la firma del documento
		        signTransaction.getDocuments().get(0).setSign(signature);
		        signTransaction.getDocuments().get(0).setId(transaction);
		        signTransaction.setId(transaction);
	        }
		}
		catch (Exception e) {
			throw new SignerException(e);
		}

	}
	
	/**
	 * Inicializa los datos para el cliente de firma de @firma
	 * @param signTransaction Transacción de firma
	 * @param request Petición
	 * @throws SignerException
	 */
	protected void initSignerClientConfig (SignTransaction signTransaction, HttpServletRequest request) throws SignerException {

		JSONArray jsonArray = new JSONArray();

		for (SignDocument document : signTransaction.getDocuments()) {
			Map<String, Object> mapConfig = new HashMap<String, Object>();
			mapConfig.put("hashToSign", document.getHash());
			mapConfig.put("transactionId", document.getId());
			mapConfig.put("algorithm", this.hashAlgoritm);
			mapConfig.put("format", this.signatureFormat);
			mapConfig.put("signData", DEFAULT_SIGN_DATA);
			mapConfig.put("certificateData", DEFAULT_CERTIFICATE_DATA);

			jsonArray.add(JSONObject.fromObject(mapConfig));
		}

        String clientSignerConfig = "";
        if (jsonArray != null) {
            clientSignerConfig = jsonArray.toString();
        }

        request.setAttribute("clientSignerConfig", clientSignerConfig) ;
     }

}
