package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.Future;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

/**
 * Clase de dominio que representa un apoderamiento o representación
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Table(name = "GRP_EMPOWERMENT")
@Audited
@NamedQueries({@NamedQuery(name = "findEmpowermentByGrantor",
		query = "from Empowerment emp where emp.idGrantor = :grantor"),
			  @NamedQuery(name = "findEmpowermentByAttorney",
		query = "from Empowerment emp where emp.idAttorney = :attorney") })
		@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "Empowerment", propOrder = {
    "empowermentDocuments",
    "empowermentNumber",
    "empowermentStates",
    "empowermentType",
    "grantingType",
    "id",
    "idAttorney",
    "idGrantor",
    "procedure",
    "selected",
    "typeGrantingDesc",
    "validityEndDate",
    "validityEndDateDesc",
    "validityInitDate",
    "validityInitDateDesc"
})
public class Empowerment implements Serializable {

    /**
     * serial version uid
     */
    private static final long serialVersionUID = -8330253913239864925L;

    /**
     * Identificador del apoderamiento
     */
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "empowerment_sequence")
    @SequenceGenerator(allocationSize = 1, name = "empowerment_sequence", sequenceName = "GRP_S_EMPOWERMENT")
    @Column(name = "X_EMPO")
    private Long id;

    /**
     * Identificador del poderdante
     */
    @NotNull
    @NotEmpty
    @Column(name = "C_GRANTOR", length = GRepresentaNumericConstants.TEXT_50)
    @XmlElement(required = true)
    private String idGrantor;

    /**
     * Identificador del apoderado
     */
    @NotNull
    @NotEmpty
    @Column(name = "C_ATTORNEY", length = GRepresentaNumericConstants.TEXT_50)
    @XmlElement(required = true)
    private String idAttorney;

    /**
     * Numero de referencia del apoderamiento
     */
    @Column(name = "T_NUM_EMP", length = GRepresentaNumericConstants.TEXT_20)
    @XmlElement(required = true)
    private String empowermentNumber;

    /**
     * Fecha de inicio de vigencia del apoderamiento
     */
    @Temporal(TemporalType.DATE)
    @Column(name = "F_INIT_VAL")
    @XmlElement(required = true)
    @XmlSchemaType(name = "date")
    private Date validityInitDate = new Date();

    /**
     * Fecha de fin de vigencia del apoderamiento
     */
    @Temporal(TemporalType.DATE)
    @Column(name = "F_END_VAL")
    @Future
    @XmlElement(required = true)
    @XmlSchemaType(name = "date")
    private Date validityEndDate;

    /**
     * Procedimiento al que pertenece el apoderamiento
     */
    @ManyToOne(fetch = FetchType.EAGER, cascade = CascadeType.ALL)
    @JoinColumn(name = "PROC_X_PROC")
    @XmlElement(required = true)
    private Procedure procedure;

    /**
     * Código del tipo de otorgamiento (global, trámites o notificaciones)
     */
    @Column(name = "V_GRANTING_TYPE", length = 1)
    @XmlElement(required = true)
    private String grantingType;

    /**
     * Descripción del tipo de otorgamiento (global, trámites o notificaciones)
     */
    @Transient
    @XmlElement(required = true)
    private String typeGrantingDesc;

    /**
     * Tipo de apoderamiento (telematico, presencial o notarial)
     */
    @Column(name = "V_EMP_TYPE", length = 1)
    @XmlElement(required = true)
    private String empowermentType;

    /**
     * Estados del apoderamiento
     */
    @OneToMany(mappedBy = "empowerment")
    private List<EmpowermentState> empowermentStates;

    /**
     * Documentos del apoderamiento
     */
    @OneToMany (mappedBy = "empowerment")
    private List<EmpowermentDocument> empowermentDocuments;

    /**
     * Indica si el apoderamiento esta seleccionado
     */
    @Transient
    private boolean selected;

    /**
     * Constructor sin parametros
     */
    public Empowerment() { }

    /**
     * Constructor con procedimiento
     * @param proc procedimiento
     * @param selected esta seleccionado
     */
    public Empowerment(Procedure proc, boolean selected) {
    	this.procedure = proc;
    	this.selected = selected;
    	this.grantingType = "T";
    }

    /**
     * Devuelve el id
     * @return id
     */
    public Long getId() {
        return id;
    }

    /**
     * Modifica el id
     * @param id nuevo id
     */
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * Devuelve el id del poderdante
     * @return id del poderdante
     */
    public String getIdGrantor() {
        return idGrantor;
    }

    /**
     * Modifica el id del poderdante
     * @param idGrantor nuevo id
     */
    public void setIdGrantor(String idGrantor) {
        this.idGrantor = idGrantor;
    }

    /**
     * Devuelve el id del apoderado
     * @return id del apoderado
     */
    public String getIdAttorney() {
        return idAttorney;
    }

    /**
     * Modifica el id del apoderado
     * @param idAttorney nuevo id del apoderado
     */
    public void setIdAttorney(String idAttorney) {
        this.idAttorney = idAttorney;
    }

    /**
     * Devuelve fecha inicio vigencia
     * @return fecha de inicio de vigencia
     */
    public Date getValidityInitDate() {
        return validityInitDate;
    }

    /**
     * Modifica fecha inicio vigencia
     * @param validityInitDate nueva fecha inicio vigencia
     */
    public void setValidityInitDate(Date validityInitDate) {
        this.validityInitDate = validityInitDate;
    }

    /**
     * Devuelve fecha fin vigencia
     * @return fecha fin de vigencia
     */
    public Date getValidityEndDate() {
    	return validityEndDate;
    }

    /**
     * Modifica fecha fin de vigencia
     * @param validityEndDate fecha fin de vigencia
     */
    public void setValidityEndDate(Date validityEndDate) {
    	this.validityEndDate = validityEndDate;
    }

    /**
     * Devuelve el procedimiento
     * @return procedimiento
     */
    public Procedure getProcedure() {
        return procedure;
    }

    /**
     * Modifica el procedimiento
     * @param procedure nuevo procedimiento
     */
    public void setProcedure(Procedure procedure) {
        this.procedure = procedure;
    }

    /**
     * Devuelve el tipo de otorgamiento
     * @return tipo de otorgamiento
     */
    public String getGrantingType() {
        return grantingType;
    }

    /**
     * Modifica el tipo de otorgamiento
     * @param grantingType nuevo tipo de otorgamiento
     */
    public void setGrantingType(String grantingType) {
        this.grantingType = grantingType;
    }

    /**
     * Devuelve el tipo de apoderamiento
     * @return tipo de apoderamiento
     */
    public String getEmpowermentType() {
        return empowermentType;
    }

    /**
     * Modifica el tipo de apoderamiento
     * @param empowermentType nuevo tipo de apoderamiento
     */
    public void setEmpowermentType(String empowermentType) {
        this.empowermentType = empowermentType;
    }

    /**
     * Devuelve el estado del apoderamiento
     * @return estado del apoderamiento
     */
	public List<EmpowermentState> getEmpowermentStates() {
		return empowermentStates;
	}

	/**
	 * Modifica el estado del apoderamiento
	 * @param empowermentStates nuevo estado del apoderamiento
	 */
	public void setEmpowermentStates(List<EmpowermentState> empowermentStates) {
		this.empowermentStates = empowermentStates;
	}

	/**
	 * Devuelve la lista de documentos de un apoderamiento
	 * @return lista de documentos de un apoderamiento
	 */
	public List<EmpowermentDocument> getEmpowermentDocuments() {
		return empowermentDocuments;
	}

	/**
	 * Modifica la lista de documentos de un apoderamiento
	 * @param empowermentDocuments nueva lista de documentos de un apoderamiento
	 */
	public void setEmpowermentDocuments(
			List<EmpowermentDocument> empowermentDocuments) {
		this.empowermentDocuments = empowermentDocuments;
	}

	/**
	 * Modifica si esta seleccionado
	 * @param selected si esta o no
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * Indica si esta seleccionado
	 * @return si esta o no
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * Devuelve la descripcion del tipo de otorgamiento
	 * @return descripcion tipo otorgamiento
	 */
	public String getTypeGrantingDesc() {
		if ("G".equals(grantingType)) {
			return "Global";
		} else if ("N".equals(grantingType)) {
			return "Notificaciones";
		} else {
			return "Trámites";
		}
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	@XmlElement(required = true)
	public String getValidityEndDateDesc() {
		if (validityEndDate != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(validityEndDate);
		} else {
			return "";
		}
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	@XmlElement(required = true)
	public String getValidityInitDateDesc() {
		if (validityInitDate != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(validityInitDate);
		} else {
			return "";
		}
	}

	/**
	 * Modifica el numero de apoderamiento
	 * @param empowermentNumber nuevo numero de apoderamiento
	 */
	public void setEmpowermentNumber(String empowermentNumber) {
		this.empowermentNumber = empowermentNumber;
	}

	/**
	 * Devuelve el numero de apoderamiento
	 * @return numero de apoderamiento
	 */
	public String getEmpowermentNumber() {
		return empowermentNumber;
	}

	/**
	 * Modifica la descripcion del tipo
	 * @param typeGrantingDesc nuevo tipo
	 */
	public void setTypeGrantingDesc(String typeGrantingDesc) {
		this.typeGrantingDesc = typeGrantingDesc;
	}


}
