package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.impl;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;

import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.DocumentsManager;
import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.DocumentsManagerFactory;
import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.exception.DocumentsManagerException;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Document;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentState;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Procedure;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.State;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.EmpowermentQueryServiceDAO;


/**
 * Implementación del servicio que gestiona los apoderamientos
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("empowermentService")
@Scope(ScopeType.SESSION)
public class EmpowermentQueryServiceDAOImpl implements EmpowermentQueryServiceDAO, Serializable {

    /**
     * Serial
     */
    private static final long serialVersionUID = -6879950992822775858L;

    /**
     * Log
     */
    @Logger
    private Log log;

    /**
     * EntityManager
     */
    @In
    private EntityManager entityManager;

    /**
     * Mapa de estados para no tener que consultar
     */
    @In(required = false, scope = ScopeType.APPLICATION)
    private Map<String,State> states;

    /**
     *  configuration
     */
    @In
    private Properties configurationgr;

    /**
     * Crea un apoderamiento
     * @param empowerment apoderamiento
     * @param documentId id del documento
     * @param empoDocument documento apoderamiento
     * @return si se crea o no
     */
    public boolean createEmpowerment(Empowerment empowerment, Long documentId , EmpowermentDocument empoDocument) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method createEmpowerment...");
        }

        boolean create = false;

        if (empowerment != null || documentId != null) {

            Procedure proc = entityManager.merge(empowerment.getProcedure());
            empowerment.setProcedure(proc);

            if (empowerment.getId() != null) {
    			entityManager.merge(empowerment);
    		} else {
    			entityManager.persist(empowerment);
    		}

            empowerment.setEmpowermentNumber(empowerment.getId() + "/" + new GregorianCalendar().get(Calendar.YEAR));

            //Relacionamos el apoderamiento con el documento
            createEmpowermentDocument(empowerment, documentId, empoDocument);

            //Establecemos el estado inicial
            EmpowermentState empowermentState = new EmpowermentState();
            empowermentState.setEmpowerment(empowerment);
            empowermentState.setInitDate(new Date());
            //Si el tipo de presentación es telemática y el procedimiento requiere confirmación
            if (GRepresentaConstants.EMP_TYPE_TELEMATIC.equals(empowerment.getEmpowermentType())
	    		&& (!GRepresentaConstants.GRANTING_PROCEDURE.equals(empowerment.getGrantingType()) || proc.isConfirmEmpowerment())) {
                empowermentState.setState(states.get(GRepresentaConstants.STATE_PENDING));
            } else {
                empowermentState.setState(states.get(GRepresentaConstants.STATE_CONFIRM));
            }

	    	if (empowermentState.getId() != null) {
	    		entityManager.merge(empowermentState);
	    	} else {
            entityManager.persist(empowermentState);
	    	}

            entityManager.flush();

            create = true;

        } else {
            throw new IllegalArgumentException("Empowerment or document is null");
        }

        return create;
    }

    /**
     * Crea un documento
     * @param originalDocument documento a crear
     * @throws DocumentsManagerException content exception
     */
    public void createDocument(Document originalDocument) throws DocumentsManagerException {

        if (log.isDebugEnabled()) {
            log.debug("Calling method createDocument...");
        }

        Document document = new Document();
		try {
			document = (Document) originalDocument.clone();
		} catch (CloneNotSupportedException e) {
			log.error(e.getMessage(), e);
		}

        if (document.getId() != null) {
    		entityManager.merge(document);
    	} else {
    		entityManager.persist(document);
    	}
        
        document.setName(document.getName().replace(".pdf", "") + "_" + document.getId() + ".pdf");
        
        String useDocumentsManager = (String) configurationgr.get(GRepresentaConstants.DOCUMENT_MANAGER_USE);

        if (useDocumentsManager != null && useDocumentsManager.length() > 0
        		&& "true".equalsIgnoreCase(useDocumentsManager)) {

        	DocumentsManager docManager = DocumentsManagerFactory.createInstance();

        	SignDocument documentContent = new SignDocument();

        	documentContent.setId(document.getId().toString());
        	documentContent.setContent(document.getDocument());
        	documentContent.setMimeType(document.getFormat());

        	if (document.getName() != null && document.getName().length() > 0) {
        		documentContent.setName(document.getName());
        	}
        	else {
        		documentContent.setName("document.pdf");
        	}

        	//datos de firma
        	if (document.getTransactionCode() != null && document.getTransactionCode().length() > 0) {
        		documentContent.setSign(document.getTransactionCode());
        	}

        	docManager.save(documentContent);

        	document.setDocument(new byte[0]);
			document.setExtCode(documentContent.getId());
        }

        entityManager.flush();
        //Identificador porque esta persistido
        originalDocument.setId(document.getId());

    }

    /**
     * Crea un documento para un apoderamiento
     * @param empowerment apoderamiento
     * @param documentId id del documento
     * @param empoDocument documento apoderamiento
     * @return si se ha creado o no
     */
    public boolean createEmpowermentDocument(Empowerment empowerment,
            Long documentId, EmpowermentDocument empoDocument) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method createEmpowermentDocument...");
        }

        boolean create = false;

        if (empowerment != null || documentId != null) {

            if (empoDocument == null) {
                empoDocument = new EmpowermentDocument();
            }
            empoDocument.setIdEmpowerment(empowerment.getId());
            empoDocument.setIdDocument(documentId);
            entityManager.persist(empoDocument);

            entityManager.flush();

            create = true;

        } else {
            throw new IllegalArgumentException("Empowerment or document is null");
        }

        return create;
    }

    /**
     * Modifica un apoderamiento
     * @param empowerment apoderamiento
     * @return si se modifica o no
     */
    public boolean updateEmpowerment(Empowerment empowerment) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method updateEmpowerment...");
        }

        boolean update = false;

        if (empowerment != null) {

            entityManager.merge(empowerment);
            entityManager.flush();
            update = true;

        } else {
            throw new IllegalArgumentException("Empowerment is null");
        }

        return update;
    }

    /**
     * Devuelve la lista de apoderamientos para el apoderado indicado
     * @param idAttorney id del apoderado
     * @return lista de apoderamientos
     */
    public List<Empowerment> getEmpowermentAsAttorney(String idAttorney) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getEmpowermentAsAttorney...");
        }

        return entityManager.createNamedQuery("findEmpowermentByAttorney").setParameter("attorney", idAttorney).getResultList();
    }

    /**
     * Devuelve la lista de apoderamientos para el poderdante indicado
     * @param idGrantor id del poderdante
     * @return lista de apoderamientos
     */
    public List<Empowerment> getEmpowermentsAsGrantor(String idGrantor) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getEmpowermentsAsGrantor...");
        }

        return entityManager.createNamedQuery("findEmpowermentByGrantor").setParameter("grantor", idGrantor).getResultList();
    }

    /**
     * Historico de estados para un apoderamiento
     * @param idEmpowerment id del apoderamiento
     * @return listado de estados
     */
    public List<EmpowermentState> getStatesForEmpowerment(Long idEmpowerment) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getStatesForEmpowerment...");
        }

        Query query = entityManager.createNamedQuery("findStatesForEmpowerment");
        query.setParameter("idEmpowerment", idEmpowerment);
        query.setParameter("now", new Date());
        return query.getResultList();
    }

    /**
     * Documentos de un apoderamiento
     * @param idEmpowerment id del apoderamiento
     * @return listado de documentos
     * @throws DocumentsManagerException content exception
     */
    public List<EmpowermentDocument> getDocumentsForEmpowerment(Long idEmpowerment) throws DocumentsManagerException {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getDocumentsForEmpowerment...");
        }

        List<EmpowermentDocument> documents =
        		entityManager.createNamedQuery("findDocumentsForEmpowerment").setParameter("idEmpowerment", idEmpowerment).getResultList();
        if (documents != null) {

        	 String useDocumentsManager = (String) configurationgr.get(GRepresentaConstants.DOCUMENT_MANAGER_USE);

             if (useDocumentsManager != null && useDocumentsManager.length() > 0
             		&& "true".equalsIgnoreCase(useDocumentsManager)) {

        	 DocumentsManager docManager = DocumentsManagerFactory.createInstance();

	        	for (EmpowermentDocument empoDoc : documents) {
	        		Document document = empoDoc.getDocument();
					if (document.getExtCode() != null && document.getExtCode().length() > 0) {

						SignDocument documentContent = docManager.get(document.getExtCode());

						document.setDocument(documentContent.getContent());
					}
				}
             }
        }

        return documents;
    }

    /**
     * Modifica el estao anterior y lo pasa a uno nuevo
     * @param empowermentStateOld estado anterior
     * @param empowermentStateNew estado nuevo
     * @return si se ha realizado la operacion de forma correcta
     */
    public boolean updateState(EmpowermentState empowermentStateOld,
            EmpowermentState empowermentStateNew) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method updateState...");
        }

        boolean updated = false;

        //Modificamos el antiguo
        Date current = new Date();

        empowermentStateOld.setEndDate(current);
        entityManager.merge(empowermentStateOld);

        //Creamos el nuevo
        empowermentStateNew.setInitDate(current);

        if (empowermentStateNew.getId() != null) {
			entityManager.merge(empowermentStateNew);
		} else {
			entityManager.persist(empowermentStateNew);
		}

        entityManager.flush();

        updated = true;

        return updated;
    }

    /**
     * Estados de todos los apoderamientos
     * @return listado de estados
     */
    public Map<Long, State> getCurrentStateForAllEmpowerment() {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getCurrentStateForAllEmpowerment...");
        }

        Map<Long,State> states = new HashMap<Long, State>();
        List<EmpowermentState> allStates = null;

        Query query = entityManager.createNamedQuery("findCurrentStateFotAllEmpowerments");
        query.setParameter("now", new Date());

        allStates = query.getResultList();

        for (EmpowermentState empowermentState : allStates) {
            states.put(empowermentState.getEmpowerment().getId(), empowermentState.getState());
        }

        return states;
    }

    /**
     * Devuelve todos los estados
     * @return estados
     */
    public List<State> getAllStates() {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getAllStates...");
        }

        return entityManager.createNamedQuery("findAllState").getResultList();
    }

    /**
     * Apoderamientos relacionados con uno dado
     * @param idEmpowerment id del apoderamiento
     * @param idDocument id del documento
     * @return listado de documentos
     */
    public List<Empowerment> getRelatedEmpowerments(Long idEmpowerment,
            Long idDocument) {

        if (log.isDebugEnabled()) {
            log.debug("Calling method getRelatedEmpowerments...");
        }

        Query query = entityManager.createNamedQuery("findRelatedEmpowerments");
        query.setParameter("idEmpowerment", idEmpowerment);
        query.setParameter("idDocument", idDocument);
        return query.getResultList();
    }

    /**
     * get log
     * @return log
     */
    public Log getLog() {
        return log;
    }

    /**
     * set log
     * @param log log
     */
    public void setLog(Log log) {
        this.log = log;
    }

    /**
     * get entityManager
     * @return entityManager
     */
    public EntityManager getEntityManager() {
        return entityManager;
    }

    /**
     * set entityManager
     * @param entityManager entityManager
     */
    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

     /**
      * get the states
     * @return the states
     */
    public Map<String, State> getStates() {
        return states;
    }

    /**
     * set the states
     * @param states the states to set
     */
    public void setStates(Map<String, State> states) {
        this.states = states;
    }

	/**
	 * return the configurationgr
	 * @return the configurationgr
	 */
	public Properties getConfigurationgr() {
		return configurationgr;
	}

	/**
	 * set the configurationgr
	 * @param configurationgr the configurationgr to set
	 */
	public void setConfigurationgr(Properties configurationgr) {
		this.configurationgr = configurationgr;
	}

}
