package edu.uoc.tfc.j2ee.icampanario.grepresenta.telematic.actions;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;
import org.jboss.seam.security.Identity;

import edu.uoc.tfc.j2ee.icampanario.components.authenticator.Authenticator;
import edu.uoc.tfc.j2ee.icampanario.components.authenticator.AuthenticatorFactory;
import edu.uoc.tfc.j2ee.icampanario.components.authenticator.exception.AuthenticatorException;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManager;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManagerFactory;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.exception.CitizensManagerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Procedure;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.ProcedureQueryServiceDAO;


/**
 * Action para la autenticación del usuario
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("authenticator")
@Scope(ScopeType.SESSION)
public class AuthenticatorAction implements Serializable {

	/**
	 * Serial version uid
	 */
	private static final long serialVersionUID = 1912577200178270807L;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * Componente de autenticacion
	 */
	@In
	private Identity identity;

	/**
	 * Usuario a establecer
	 */
	@Out(required = false, scope = ScopeType.SESSION)
	private User user;

	/**
	 * CitizensManager
	 */
	@Out(required = false, scope = ScopeType.SESSION)
	private CitizensManager citizensManager;

	/**
	 * Indica si se ha autenticado el usuario
	 */
	private boolean authenticated;
	
	/**
	 * Dao de procedimientos
	 */
	@In(create = true)
	private ProcedureQueryServiceDAO procedureService;
	
	/**
	 * Lista de procedimientos
	 */
	private List<Procedure> procedures = new LinkedList<Procedure>();

	/**
	 * Si se ha autenticado
	 * @return si se ha autenticado
	 */
	public boolean isAuthenticated() {
		return authenticated;
	}

	/**
	 * Realiza la autenticación del usuario
	 * @return si se autentica o no
	 */
	public boolean authenticate() {

		log.info("Authenticating...");

		HttpServletRequest request = (HttpServletRequest) FacesContext
				.getCurrentInstance().getExternalContext().getRequest();

		authenticated = false;

		Authenticator authenticator = AuthenticatorFactory.createInstance();

		if (authenticator != null) {

			try {

				//Para rellenar el usuario autenticado con los datos del CitizensManager
				User tmp = authenticator.authenticate(request);
				String certificate = tmp.getCertificate();
				citizensManager = CitizensManagerFactory.createInstance();
				List<String> ids = new LinkedList<String>();

				ids = citizensManager.find(tmp);

				if (ids != null && ids.size() > 0) {
					user = citizensManager.get(ids.get(0));
					if (certificate != null) {
						user.setCertificate(certificate);
					}

					authenticated = true;
				}
			} catch (CitizensManagerException e) {
				log.error(e.getMessage(), e);
			} catch (AuthenticatorException e) {
				log.error(e.getMessage());
			}

		} else {
			log.error("#{messages['error.authenticatorNull']}");
		}
		return authenticated;
	}

	/**
	 * Redirige a la página de autenticacón del cliente de firma
	 * @return direccion
	 */
	public String redirect() {

		log.info("Redirecting....");

		HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		Authenticator authenticator = AuthenticatorFactory.createInstance();

		String url = null;
		if (authenticator != null) {
			try {
				url = authenticator.getAuthenticationURL(request);
			} catch (AuthenticatorException e) {
				log.error(e.getMessage(), e);
			}
		}

		return url;
	}

	/**
	 * Modifica el usuario
	 * @param user nuevo usuario
	 */
	public void setUser(User user) {
		this.user = user;
	}

	/**
	 * Devuelve el usuario
	 * @return usuario
	 */
	public User getUser() {
		return user;
	}
	
	/**
	 * Carga la lista de procedimientos
	 */
	public void chargeProceduresList() {
		log.info("Getting procedures for presential...");

		procedures = procedureService.getProcedures();
	}

	public List<Procedure> getProcedures() {
		return procedures;
	}

	public void setProcedures(List<Procedure> procedures) {
		this.procedures = procedures;
	}

}
