package edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.office;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.imageio.ImageIO;

import net.sf.jooreports.opendocument.OpenDocumentIO;
import net.sf.jooreports.templates.DocumentTemplate;
import net.sf.jooreports.templates.ZippedDocumentTemplate;

import org.jdom.Attribute;
import org.jdom.Content;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.Text;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import com.artofsolving.jodconverter.DocumentConverter;
import com.artofsolving.jodconverter.DocumentFormat;
import com.artofsolving.jodconverter.DocumentFormatRegistry;
import com.artofsolving.jodconverter.XmlDocumentFormatRegistry;
import com.artofsolving.jodconverter.openoffice.connection.OpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.connection.SocketOpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.converter.StreamOpenOfficeDocumentConverter;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;
import freemarker.ext.beans.BeansWrapper;
import freemarker.ext.beans.MapModel;
import freemarker.template.Configuration;

/**
 * Clase de utilidad para el tratamiendo de las plantillas OpenOffice
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class OfficeTools {

	//Constantes
	private static final String FILE_CONTENT_XML = "content.xml";
	private static final String XML_ENCODING = "UTF-8";

	//Elements
	private static final String ELEM_AUTOMATIC_STYLE = "automatic-styles";

	//Uris y Namespaces
	private static final String URI_TABLE = "urn:oasis:names:tc:opendocument:xmlns:table:1.0";
	private static final Namespace NAMESPACE_TABLE = Namespace.getNamespace("table", URI_TABLE);

	private static final String URI_OFFICE = "urn:oasis:names:tc:opendocument:xmlns:office:1.0";
	private static final Namespace NAMESPACE_OFFICE = Namespace.getNamespace("office", URI_OFFICE);

	private static final String URI_STYLE = "urn:oasis:names:tc:opendocument:xmlns:style:1.0";
	private static final Namespace NAMESPACE_STYLE = Namespace.getNamespace("style", URI_STYLE);

	private static final String URI_TEXT = "urn:oasis:names:tc:opendocument:xmlns:text:1.0";

	private static final String URI_FO = "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0";
	private static final Namespace NAMESPACE_FO = Namespace.getNamespace("fo", URI_FO);

	private static final String URI_DRAW = "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0";
	private static final Namespace NAMESPACE_DRAW = Namespace.getNamespace("draw", URI_DRAW);

	private static final String URI_XLINK = "http://www.w3.org/1999/xlink";
	private static final Namespace NAMESPACE_XLINK = Namespace.getNamespace("xlink", URI_XLINK);

	private static final String URI_SVG = "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0";
	private static final Namespace NAMESPACE_SVG = Namespace.getNamespace("svg", URI_SVG);

	//Variables
	private Map variables = new HashMap();
	private Map images = new HashMap();

	//Atributos
	private String markVariables = null;
	private String markVariablesEscaped = null;
	private String prefixTable = null;
	private String prefixImage = null;
	private String separatorRows = null;
	private String separatorCols = null;

	/**
	 * Constructor
	 */
	public OfficeTools() {
		this(GRepresentaConstants.MARK_VARIABLES_DEFAULT, GRepresentaConstants.PREFIX_TABLE_DEFAULT, GRepresentaConstants.PREFIX_IMAGE_DEFAULT, GRepresentaConstants.SEPARATOR_ROWS_DEFAULT, GRepresentaConstants.SEPARATOR_COLS_DEFAULT);
	}

	/**
	 * Constructor
	 * @param markVariables Marca para las variables
	 * @param prefixTable Prefijo para tablas
	 * @param prefixImage Prefijo para imágenes
	 * @param separatorRows Separador de columnas
	 * @param separatorCols Separador de filas
	 */
	public OfficeTools(String markVariables, String prefixTable,
			String prefixImage, String separatorRows, String separatorCols) {
		super();
		this.markVariables = markVariables;
		this.markVariablesEscaped = markVariables.replaceAll("([\\\\*+\\[\\](){}\\$.?\\^|])", "\\\\$1");
		this.prefixTable = prefixTable;
		this.prefixImage = prefixImage;
		this.separatorRows = separatorRows;
		this.separatorCols = separatorCols;
	}

	/**
	 * Reemplaza las variables de documento OpenOffice
	 * @param openDocument Documento de OpenOffice tomado como referencia
	 * @param vars Mapa de variables y valores
	 * @param finalDocument Documento de OpenOffice con las variables sustituidas
	 * @throws IOException
	 * @throws JDOMException
	 */
	public void replaceVars(InputStream openDocument, Map vars, OutputStream finalDocument) throws IOException, JDOMException {

		if (vars != null) {
			this.variables = vars;
		}

		ZipOutputStream zipOut = new ZipOutputStream(finalDocument);

		ZipInputStream zipFile = new ZipInputStream(openDocument);
		ZipEntry entry;

		//Recorremos todas las entradas del zip, si es el content.xml
		//sustituimos las variables, si no añadimos al zip final
		while ((entry = zipFile.getNextEntry()) != null) {
			if (entry.getName().equals(FILE_CONTENT_XML)) {

				//Guardamos el documento en un OutputStream ya que si usamos
				//directamente el zipFile al hacer el sax.build cierra el stream
				ByteArrayOutputStream baOut = new ByteArrayOutputStream();

				GRepresentaUseful.copyDataStream(zipFile, baOut);

				//Parseamos el content.xml para buscar las variables y sustituirlas
				SAXBuilder sax = new SAXBuilder();
				Document doc = sax.build(new ByteArrayInputStream(baOut.toByteArray()));
				Element rootElement = doc.getRootElement();

				Element parent = this.copyElement(rootElement);

				processElement(rootElement,parent);

				//Tenemos el documento xml con las variables sustituidas, lo añadimos al zip
				Document d = new Document(parent);

				ByteArrayOutputStream aux = new ByteArrayOutputStream();
				XMLOutputter formato = new XMLOutputter(Format.getRawFormat().setEncoding(XML_ENCODING));
				// se escribe el XML en el flujo de salida
				formato.output(d, aux);

				zipOut.putNextEntry(new ZipEntry(FILE_CONTENT_XML));
				zipOut.write(aux.toByteArray());
			}
			else {
				// Copy entry from original file to temporary file without changing anything
				ZipEntry newEntry = new ZipEntry(entry);
				newEntry.setCompressedSize(-1);  // Don't know compressed size
				zipOut.putNextEntry(newEntry);
				
				GRepresentaUseful.copyDataStream(zipFile, zipOut);

			}
		}

		//Añadimos las imágenes de las variables al zip
		addImagesToZip(zipOut);

		zipOut.close();
		zipFile.close();
	}


	/**
	 * Procesa los elementos xml del fichero content.xml
	 * @param o Elemento a procesar
	 * @param parent Elemento padre
	 */
	private void processElement(Object o, Element parent) {

		if (o instanceof Element) {

			Element e = (Element) o;

			List children = e.getContent();

			Iterator iterator = children.iterator();

			while (iterator.hasNext()) {

				Object child = iterator.next();
				Element el = null;

				if (child instanceof Text) {
					Text t = (Text)child;
					String sContenidoTexto = t.getText();
					Content varResult = null;

					if (sContenidoTexto.length() > 0) {
						varResult = parseTextVariables(sContenidoTexto);
					}

					if (varResult != null) {
						if (varResult instanceof Element && ((Element) varResult).getName().equals("table")) {
							parent.getParentElement().addContent(varResult);
						}
						else {
							parent.addContent(varResult);
						}
					}
				}
				else {
					el = (Element)child;
					Element newParent = this.copyElement(el);

					if (newParent.getName().equals(ELEM_AUTOMATIC_STYLE)) {
						addStyles(newParent);
					}

					parent.addContent(newParent);
					processElement(child,newParent);
				}
			}
		}
	}

	/**
	 * Reemplaza las variables en el texto indicado. Si la variable es una imagen
	 * o tabla se llama al método encargado de hacer la sustitución para esos
	 * casos.
	 * @param text Texto del párrafo al que se van a sustituir las variables
	 * @return Devuelve un elemento para el xml que puede ser el texto con las
	 * variable sustituidas, el elemento tabla o una imágen.
	 */
	private Content parseTextVariables(String text) {

		try {

			int nIni = text.indexOf(this.markVariables);
			int nFin = 0;

			while (nIni >= 0) {
				nFin = text.indexOf(this.markVariables, nIni + 2);
				if (nFin != -1) {
					String var = text.substring(nIni + 2, nFin);

					String valorVar = (String) variables.get(var);

					if (variables.containsKey(var) && (valorVar == null || valorVar.length() < 1)) {
						valorVar = GRepresentaUseful.getErrorVar(var) ;
						text = text.replaceAll(this.markVariablesEscaped + var + this.markVariablesEscaped, valorVar);
					}
					else {

						//Si la variable existía pero ha devuelto null o
						//cadena vacía dejamos la variable
						if (valorVar == null || valorVar.length() < 1) {
							text = this.markVariables + var + this.markVariables;
						}
						else {
							if (var.startsWith(this.prefixTable)) {
								return parseVarTable(var, valorVar);
							}
							else if (var.startsWith(this.prefixImage)) {

								return parseVarImage(var, valorVar);

							}
							else {
								text = text.replaceAll(this.markVariablesEscaped + var + this.markVariablesEscaped, valorVar);
							}
						}

					}

					nIni = text.indexOf(this.markVariables, nFin + 2);
				}
				else {
					nIni = -1;
				}
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
		return new Text(text);

	}

	/**
	 * Crea un nuevo elemento tabla que sustituirá a la variable indicada.
	 * @param var Nombre de la variable
	 * @param valueVar Valor de la variable con los separadores de fila y columna
	 * a partir del cuál se montará el elemento xml para el documento final.
	 * @return Element tabla para el content.xml
	 */
	private Element parseVarTable(String var, String valueVar) {
		Element elementTable = new Element("table","table",URI_TABLE);
		elementTable.setAttribute("name", var, Namespace.getNamespace("table", URI_TABLE));

		int i = 0;
		StringTokenizer stkRow = new StringTokenizer(valueVar,this.separatorRows);

		while (stkRow.hasMoreElements()) {
			String rowValue = (String) stkRow.nextElement();

			Element elementRow = new Element("table-row","table",URI_TABLE);

			StringTokenizer stkColumn = new StringTokenizer(rowValue,this.separatorCols);
			int numColumns = stkColumn.countTokens();

			if (i == 0) {
				//La primera vez añadimos los atributos para el número de columnas
				Element elementTableColumn = new Element("table-column","table",URI_TABLE);
				elementTableColumn.setAttribute("number-columns-repeated",String.valueOf(numColumns), NAMESPACE_TABLE);
				elementTable.addContent(elementTableColumn);
			}

			while (stkColumn.hasMoreElements()) {
				String columnValue = (String) stkColumn.nextElement();

				Element elementCell = new Element("table-cell","table",URI_TABLE);
				elementCell.setAttribute("value-type", "string", NAMESPACE_OFFICE);
				elementCell.setAttribute("style-name", "TrTableVar", NAMESPACE_TABLE);

				Element elementTextCell = new Element("p", "text", URI_TEXT);
				elementTextCell.addContent(columnValue);
				elementCell.addContent(elementTextCell);

				elementRow.addContent(elementCell);
			}

			elementTable.addContent(elementRow);

			i++;

		}

		return elementTable;
	}

	/**
	 * Crea un nuevo elemento imagen que sustituirá a la variable indicada.
	 * @param var Nombre de la variable
	 * @param valueVar Valor de la variable que contendrá la ruta de la imagen
	 * @return Element imagen que contendrá el documento final en el lugar de la variable.
	 * @throws IOException
	 */
	private Element parseVarImage(String var, String valueVar) throws IOException {

		Element elementDraw = new Element("frame", "draw", URI_DRAW);
		elementDraw.setAttribute("name", var, NAMESPACE_DRAW);
		final float PIXEL_TO_CM = 0.02646f;
		URL url = new URL(valueVar);
		InputStream is = url.openStream();
		BufferedImage bufferedImage = ImageIO.read(is);
		elementDraw.setAttribute("width", String.valueOf(bufferedImage.getWidth() * PIXEL_TO_CM) + "cm", NAMESPACE_SVG);
		elementDraw.setAttribute("height", String.valueOf(bufferedImage.getHeight() * PIXEL_TO_CM) + "cm", NAMESPACE_SVG);

		Element elementImage = new Element("image", "draw", URI_DRAW);
		String sImage = "Pictures/" + var;
		elementImage.setAttribute("href", sImage, NAMESPACE_XLINK);

		elementDraw.addContent(elementImage);

		images.put(var, valueVar);

		return elementDraw;
	}


	/**
	 * Añade los estilos para que la tabla se muestre con borde y con algo de padding.
	 * @param element Elemento xml al cuál se le añadirá el nuevo estilo, definido en
	 * la constante ELEM_AUTOMATIC_STYLE.
	 */
	private void addStyles(Element element) {
		Element elementStyle = new Element("style", "style", URI_STYLE);
		elementStyle.setAttribute("name", "TrTableVar", NAMESPACE_STYLE);
		elementStyle.setAttribute("family", "table-cell", NAMESPACE_STYLE);

		Element elementStyleBorder = new Element("table-cell-properties", "style", URI_STYLE);
		elementStyleBorder.setAttribute("border", "0.002cm solid #000000", NAMESPACE_FO);
		elementStyleBorder.setAttribute("padding", "0.097cm", NAMESPACE_FO);

		elementStyle.addContent(elementStyleBorder);

		element.addContent(elementStyle);
	}

	/**
	 * Añade las imágenes al zip final que será el documento.
	 * @param zipOut OpenDocument
	 * @throws IOException
	 */
	private void addImagesToZip(ZipOutputStream zipOut) throws IOException {
		if (!images.isEmpty()) {

			Iterator itImages = images.keySet().iterator();
			while (itImages.hasNext()) {
				String key = (String) itImages.next();

				String valueVar = (String) images.get(key);

				ZipEntry entryImage = new ZipEntry("Pictures/" + key);
				zipOut.putNextEntry(entryImage);
				URL url = new URL(valueVar);
				InputStream inputImage = url.openStream();
				GRepresentaUseful.copyDataStream(inputImage, zipOut);
			}
		}
	}

	/**
	 * Realiza la copia de un elemento xml del documento original al documento
	 * final.
	 * @param original Elemento xml original.
	 * @return Elemento xml copiado para el documento final.
	 */
	private Element copyElement(Element original) {
		Element copy = new Element(original.getName(),original.getNamespacePrefix(),original.getNamespaceURI());
		copy.setNamespace(original.getNamespace());

		if (original.getAttributes().size() > 0) {

			for (int i = 0; i < original.getAttributes().size(); i++) {
				Attribute att = (Attribute) original.getAttributes().get(i);
				copy.setAttribute(att.getName(), att.getValue(), att.getNamespace());
			}
		}

		if (original.getAdditionalNamespaces().size() > 0) {

			for (int i = 0; i < original.getAdditionalNamespaces().size(); i++) {
				Namespace ns = (Namespace) original.getAdditionalNamespaces().get(i);
				copy.addNamespaceDeclaration(ns);
			}
		}

		return copy;
	}

	/**
	 * Permite convertir un documento odt en pdf. Para ello hace uso de la librería JODConverter.
	 * @param inputOpenDocument OpenDocumento odt
	 * @param outputPdf Documento pdf
	 * @throws Exception
	 */
	public static void convertToPdf(String host, int port, InputStream inputOpenDocument, String inputFormat, OutputStream outputPdf, String outputFormat) throws Exception {

		// crea la conexión con el OpenOffice servidor
		OpenOfficeConnection connection = new SocketOpenOfficeConnection(host, port);
		connection.connect();

		// realiza la conversión de formato
		DocumentFormatRegistry registry = new XmlDocumentFormatRegistry();
		DocumentFormat formatin = registry.getFormatByFileExtension(inputFormat);
		DocumentFormat formatout = registry.getFormatByFileExtension(outputFormat);

		DocumentConverter converter = new StreamOpenOfficeDocumentConverter(connection, registry);
		converter.convert(inputOpenDocument, formatin, outputPdf, formatout);

		// cierra la conexión con el servidor
		connection.disconnect();
	}

	/**
	 * Devuelve la lista de variables que hay en el documento.
	 * Usando la marca de variables indicada en el constructor o si
	 * no se ha indicado usa $$ por defecto
	 * @param openDocument Documento OpenOffice
	 * @return Lista de variables
	 * @throws Exception Exception
	 */
    public List getVariables(InputStream openDocument) throws Exception {

        //Unzip the openOffice Document
    	ZipInputStream zipFile = new ZipInputStream(openDocument);
        ZipEntry entry;
        List vars = new LinkedList();

        while ((entry = zipFile.getNextEntry()) != null) {

            if (entry.getName().equals(FILE_CONTENT_XML)) {

            	ByteArrayOutputStream baOut = new ByteArrayOutputStream();

            	GRepresentaUseful.copyDataStream(zipFile, baOut);

            	SAXBuilder sax = new SAXBuilder();
				Document doc = sax.build(new ByteArrayInputStream(baOut.toByteArray()));
                Element rootElement = doc.getRootElement();
                processElementGetVariables(rootElement, vars, this.markVariables);
                break;
            }
        }
        return vars;
    }

    /**
     * Busca en todos los elementos del fichero conten.xml las variables
     * @param o Elemento xml
     * @param vars Lista de variables que se va actualizando
     */
    private void processElementGetVariables(Object o, List vars, String markVariables) {

        if (o instanceof Element) {

            Element e = (Element) o;
            String elementName = e.getQualifiedName();

            //Si es un elemento de texto buscamos si tiene variables
            if (elementName.startsWith("text")) {

                List children = e.getContent();
                Iterator iterator = children.iterator();

                while (iterator.hasNext()) {

                    Object child = iterator.next();

                    if (child instanceof Text) {
                        Text t = (Text) child;
                        String text = t.getValue();

                        int nIni = text.indexOf(markVariables);
            			int nFin = 0;

            			while (nIni >= 0) {
            				nFin = text.indexOf(markVariables, nIni + 2);
            				if (nFin != -1) {
            					String var = text.substring(nIni + 2, nFin);
            					nIni = text.indexOf(markVariables, nFin + 2);

            					vars.add(var);
            				}
            				else {
            					nIni = -1;
            				}
            			}
                    }
                    else
                    processElementGetVariables(child, vars, markVariables);
                }
            }
            else {
                List non_text_list = e.getContent();
                Iterator it = non_text_list.iterator();
                while (it.hasNext()) {
                    Object non_text_child = it.next();
                    processElementGetVariables(non_text_child, vars, markVariables);
                }
            }
        }
    }
    
    /**
     * 
     * @param host
     * @param port
     * @param sourceDoc
     * @param varsMap
     * @return
     * @throws Exception
     */
    public byte[] getFreemarkerVars(String host, int port, InputStream sourceDoc, Map<String, Object> varsMap) throws Exception{

    	Configuration freemarkerConfiguration = new Configuration();		

		freemarkerConfiguration.setDefaultEncoding(OpenDocumentIO.UTF_8.name());
		freemarkerConfiguration.setOutputEncoding(OpenDocumentIO.UTF_8.name());
    	
		// Plantilla
		DocumentTemplate template = new ZippedDocumentTemplate(sourceDoc, freemarkerConfiguration);

		MapModel mapafree = new MapModel(varsMap, new BeansWrapper());

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		template.createDocument(mapafree, baos);

		return baos.toByteArray();
	}
}
